package betterwithmods.common;

import betterwithmods.common.blocks.*;
import betterwithmods.common.items.ItemBark;
import betterwithmods.common.items.ItemMaterial;
import betterwithmods.common.registry.variants.WoodVariant;
import betterwithmods.library.common.variants.BlockVariant;
import betterwithmods.library.common.variants.IBlockVariants;
import betterwithmods.library.utils.InventoryUtils;
import betterwithmods.library.utils.VariantUtils;
import com.google.common.collect.HashMultimap;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemFishFood;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.CraftingManager;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.util.NonNullList;
import net.minecraftforge.oredict.OreDictionary;
import net.minecraftforge.oredict.OreIngredient;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static betterwithmods.library.common.variants.IBlockVariants.EnumBlock.*;
import static betterwithmods.library.common.variants.IBlockVariants.EnumBlock.LOG;
import static net.minecraft.init.Blocks.*;

/**
 * Created by primetoxinz on 5/10/17.
 */
public class BWMOreDictionary {

    public static final Set<IRecipe> logRecipes = Sets.newHashSet();
    public static final HashMultimap<String, String> toolEffectiveOre = HashMultimap.create();
    public static List<Ore> nuggetNames;
    public static List<Ore> dustNames;
    public static List<Ore> oreNames;
    public static List<Ore> ingotNames;
    public static List<ItemStack> logs;

    public static void registerOres() {

        toolEffectiveOre.putAll("axe", Lists.newArrayList("logWood", "plankWood"));
        toolEffectiveOre.putAll("mattock", Lists.newArrayList("stone", "cobblestone"));

        registerOre("wool", new ItemStack(Blocks.field_150325_L, OreDictionary.WILDCARD_VALUE));
        registerOre("scroll", BWMItems.ARCANE_SCROLL);
        registerOre("dung", ItemMaterial.getStack(ItemMaterial.EnumMaterial.DUNG));
        registerOre("padding", ItemMaterial.getStack(ItemMaterial.EnumMaterial.PADDING));
        registerOre("soap", ItemMaterial.getStack(ItemMaterial.EnumMaterial.SOAP));
        registerOre("archimedesScrew", ItemMaterial.getStack(ItemMaterial.EnumMaterial.SCREW));
        registerOre("filament", ItemMaterial.getStack(ItemMaterial.EnumMaterial.FILAMENT));
        registerOre("element", ItemMaterial.getStack(ItemMaterial.EnumMaterial.ELEMENT));
        registerOre("latchRedstone", ItemMaterial.getStack(ItemMaterial.EnumMaterial.REDSTONE_LATCH));
        registerOre("arrowSoulforgedSteel", ItemMaterial.getStack(ItemMaterial.EnumMaterial.BROADHEAD));
        registerOre("gearWood", ItemMaterial.getStack(ItemMaterial.EnumMaterial.WOODEN_GEAR));
        registerOre("cropHemp", ItemMaterial.getStack(ItemMaterial.EnumMaterial.HEMP_LEAF));
        registerOre("dyeBrown", ItemMaterial.getStack(ItemMaterial.EnumMaterial.DUNG));
        registerOre("dung", ItemMaterial.getStack(ItemMaterial.EnumMaterial.DUNG));
        registerOre("slimeball", ItemMaterial.getStack(ItemMaterial.EnumMaterial.GLUE));
        registerOre("glue", ItemMaterial.getStack(ItemMaterial.EnumMaterial.GLUE));
        registerOre("ingotSoulforgedSteel", ItemMaterial.getStack(ItemMaterial.EnumMaterial.STEEL_INGOT));
        registerOre("dustNetherrack", ItemMaterial.getStack(ItemMaterial.EnumMaterial.GROUND_NETHERRACK));
        registerOre("dustHellfire", ItemMaterial.getStack(ItemMaterial.EnumMaterial.HELLFIRE_DUST));
        registerOre("dustSoul", ItemMaterial.getStack(ItemMaterial.EnumMaterial.SOUL_DUST));
        registerOre("ingotConcentratedHellfire", ItemMaterial.getStack(ItemMaterial.EnumMaterial.CONCENTRATED_HELLFIRE));
        registerOre("dustCoal", ItemMaterial.getStack(ItemMaterial.EnumMaterial.COAL_DUST));
        registerOre("dustPotash", ItemMaterial.getStack(ItemMaterial.EnumMaterial.POTASH));
        registerOre("dustWood", ItemMaterial.getStack(ItemMaterial.EnumMaterial.SAWDUST));
        registerOre("dustSulfur", ItemMaterial.getStack(ItemMaterial.EnumMaterial.BRIMSTONE));
        registerOre("dustSaltpeter", ItemMaterial.getStack(ItemMaterial.EnumMaterial.NITER));
        registerOre("nuggetSoulforgedSteel", ItemMaterial.getStack(ItemMaterial.EnumMaterial.STEEL_NUGGET));
        registerOre("foodFlour", BlockRawPastry.getStack(BlockRawPastry.Type.BREAD));
        registerOre("dustCharcoal", ItemMaterial.getStack(ItemMaterial.EnumMaterial.CHARCOAL_DUST));
        registerOre("foodCocoapowder", ItemMaterial.getStack(ItemMaterial.EnumMaterial.COCOA_POWDER));
        registerOre("dustCarbon", ItemMaterial.getStack(ItemMaterial.EnumMaterial.COAL_DUST), ItemMaterial.getStack(ItemMaterial.EnumMaterial.CHARCOAL_DUST));
        registerOre("dustCoal", ItemMaterial.getStack(ItemMaterial.EnumMaterial.COAL_DUST));
        registerOre("dustCharcoal", ItemMaterial.getStack(ItemMaterial.EnumMaterial.CHARCOAL_DUST));
        registerOre("gemNetherCoal", ItemMaterial.getStack(ItemMaterial.EnumMaterial.NETHERCOAL));
        registerOre("coal", new ItemStack(Items.field_151044_h, 1, 0), new ItemStack(Items.field_151044_h, 1, 1));
        registerOre("materialNetherSludge", ItemMaterial.getStack(ItemMaterial.EnumMaterial.NETHER_SLUDGE));
        registerOre("foodChocolatebar", new ItemStack(BWMItems.CHOCOLATE));
        registerOre("chainmail", ItemMaterial.getStack(ItemMaterial.EnumMaterial.CHAIN_MAIL));

        registerOre("blockHardenedNetherClay", BlockAesthetic.getStack(BlockAesthetic.Type.NETHERCLAY));
        registerOre("blockConcentratedHellfire", BlockAesthetic.getStack(BlockAesthetic.Type.HELLFIRE));
        registerOre("blockPadding", BlockAesthetic.getStack(BlockAesthetic.Type.PADDING));
        registerOre("blockEnder", BlockAesthetic.getStack(BlockAesthetic.Type.ENDERBLOCK));
        registerOre("blockSoulforgedSteel", new ItemStack(BWMBlocks.STEEL_BLOCK, 1, 0));
        registerOre("blockBarrel", new ItemStack(BWMBlocks.BARREL));
        //Added bark subtype entries for Roots compatibility
        registerOre("barkWood", ItemBark.getBarks(1));
        registerOre("barkWood", new ItemStack(BWMItems.BARK, 1, OreDictionary.WILDCARD_VALUE));
        registerOre("barkOak", new ItemStack(BWMItems.BARK, 1, 0));
        registerOre("barkSpruce", new ItemStack(BWMItems.BARK, 1, 1));
        registerOre("barkBirch", new ItemStack(BWMItems.BARK, 1, 2));
        registerOre("barkJungle", new ItemStack(BWMItems.BARK, 1, 3));
        registerOre("barkAcacia", new ItemStack(BWMItems.BARK, 1, 4));
        registerOre("barkDarkOak", new ItemStack(BWMItems.BARK, 1, 5));
        registerOre("barkBlood", new ItemStack(BWMItems.BARK, 1, 6));

        registerOre("hideTanned", ItemMaterial.getStack(ItemMaterial.EnumMaterial.TANNED_LEATHER));
        registerOre("hideTanned", ItemMaterial.getStack(ItemMaterial.EnumMaterial.TANNED_LEATHER_CUT));
        registerOre("hideBelt", ItemMaterial.getStack(ItemMaterial.EnumMaterial.LEATHER_BELT));
        registerOre("hideScoured", ItemMaterial.getStack(ItemMaterial.EnumMaterial.SCOURED_LEATHER));
        registerOre("hideStrap", ItemMaterial.getStack(ItemMaterial.EnumMaterial.LEATHER_STRAP));
        registerOre("leather", ItemMaterial.getStack(ItemMaterial.EnumMaterial.LEATHER_CUT));

        registerOre("fiberHemp", ItemMaterial.getStack(ItemMaterial.EnumMaterial.HEMP_FIBERS));
        registerOre("fabricHemp", ItemMaterial.getStack(ItemMaterial.EnumMaterial.HEMP_CLOTH));

        registerOre("ingotDiamond", ItemMaterial.getStack(ItemMaterial.EnumMaterial.DIAMOND_INGOT));
        registerOre("nuggetDiamond", ItemMaterial.getStack(ItemMaterial.EnumMaterial.DIAMOND_NUGGET));

        registerOre("listAllmeat", Items.field_151147_al, Items.field_151082_bd, Items.field_151076_bf, Items.field_151115_aP, Items.field_179561_bm, BWMItems.MYSTERY_MEAT);
        registerOre("listAllmeat", new ItemStack(Items.field_151115_aP, 1, ItemFishFood.FishType.SALMON.func_150976_a()));
        registerOre("listAllmeatcooked", Items.field_151157_am, Items.field_151083_be, Items.field_151077_bg, Items.field_179566_aV, Items.field_179557_bn, Items.field_179559_bp, BWMItems.COOKED_MYSTERY_MEAT);
        registerOre("listAllmeatcooked", new ItemStack(Items.field_179566_aV, 1, ItemFishFood.FishType.SALMON.func_150976_a()));
        registerOre("foodStewMeat", Items.field_151157_am, Items.field_151083_be, Items.field_179566_aV, Items.field_179557_bn, BWMItems.COOKED_MYSTERY_MEAT);
        registerOre("foodStewMeat", new ItemStack(Items.field_179566_aV, 1, ItemFishFood.FishType.SALMON.func_150976_a()));

        registerOre("tallow", ItemMaterial.getStack(ItemMaterial.EnumMaterial.TALLOW));

        registerOre("pile", new ItemStack(BWMItems.DIRT_PILE), new ItemStack(BWMItems.SAND_PILE), new ItemStack(BWMItems.RED_SAND_PILE), new ItemStack(BWMItems.GRAVEL_PILE));
        registerOre("pileDirt", new ItemStack(BWMItems.DIRT_PILE));
        registerOre("pileSand", new ItemStack(BWMItems.SAND_PILE), new ItemStack(BWMItems.RED_SAND_PILE));
        registerOre("pileRedSand", new ItemStack(BWMItems.RED_SAND_PILE));
        registerOre("pileGravel", new ItemStack(BWMItems.GRAVEL_PILE));


        registerOre("treeSapling", new ItemStack(BWMBlocks.BLOOD_SAPLING));
        registerOre("treeLeaves", new ItemStack(BWMBlocks.BLOOD_LEAVES));
        registerOre("logWood", new ItemStack(BWMBlocks.BLOOD_LOG));
        registerOre("blockNetherSludge", new ItemStack(BWMBlocks.NETHER_CLAY));

        registerOre("wicker", new ItemStack(BWMBlocks.WICKER));

        registerOre("blockPlanter", BlockPlanter.getAll());
        registerOre("blockVase", BlockVase.getAll());
        registerOre("blockCandle", BlockCandle.getAll());
        registerOre("cobblestone", BlockCobble.getAll());

        registerOre("stickWood", new ItemStack(BWMBlocks.SHAFT));
        registerOre("blockWindChime", BlockChime.BLOCKS);
        registerOre("blockSoulUrn", new ItemStack(BWMBlocks.SOUL_URN));

        registerOre("dustBlaze", new ItemStack(Items.field_151065_br));

        registerOre("foodDonut", BWMItems.DONUT);

        registerOre("meatPork", Items.field_151147_al, Items.field_151157_am);
        registerOre("meatBeef", Items.field_151082_bd, Items.field_151083_be);
        registerOre("meatMutton", Items.field_179561_bm, Items.field_179557_bn);
        registerOre("meatChicken", Items.field_151076_bf, Items.field_151077_bg);
        registerOre("meatRotten", Items.field_151078_bh);
        registerOre("meatFish",
                new ItemStack(Items.field_151115_aP, 1, ItemFishFood.FishType.COD.func_150976_a()),
                new ItemStack(Items.field_151115_aP, 1, ItemFishFood.FishType.SALMON.func_150976_a()),
                new ItemStack(Items.field_179566_aV, 1, ItemFishFood.FishType.COD.func_150976_a()),
                new ItemStack(Items.field_179566_aV, 1, ItemFishFood.FishType.SALMON.func_150976_a())
        );

        registerOre("cookedPotato", Items.field_151168_bH, Items.field_151174_bG);
        registerOre("cookedCarrot", Items.field_151172_bF, BWMItems.COOKED_CARROT);


        registerOre("listAllExplosives", new ItemStack(Blocks.field_150335_W));
        registerOre("listAllExplosives", new ItemStack(Items.field_151016_H));
        registerOre("listAllExplosives", new ItemStack(BWMItems.DYNAMITE));
        registerOre("listAllExplosives", new ItemStack(BWMBlocks.MINING_CHARGE));
        registerOre("listAllExplosives", BlockAesthetic.getStack(BlockAesthetic.Type.HELLFIRE));
        registerOre("listAllExplosives", ItemMaterial.getStack(ItemMaterial.EnumMaterial.BLASTING_OIL));
        registerOre("listAllExplosives", ItemMaterial.getStack(ItemMaterial.EnumMaterial.HELLFIRE_DUST));
        registerOre("listAllExplosives", ItemMaterial.getStack(ItemMaterial.EnumMaterial.CONCENTRATED_HELLFIRE));


        registerOre("seed", new ItemStack(BWMBlocks.HEMP), new ItemStack(Items.field_151014_N), new ItemStack(Items.field_151081_bc), new ItemStack(Items.field_151080_bb), new ItemStack(Items.field_185163_cU));

        registerOre("concrete", new ItemStack(Blocks.field_192443_dR, 1, OreDictionary.WILDCARD_VALUE));
        registerOre("wool", new ItemStack(Blocks.field_150325_L, 1, OreDictionary.WILDCARD_VALUE));
        registerOre("terracotta", field_150406_ce, field_192427_dB, field_192428_dC, field_192429_dD, field_192430_dE, field_192431_dF, field_192432_dG, field_192433_dH, field_192434_dI, field_192435_dJ, field_192436_dK, field_192437_dL, field_192438_dM, field_192439_dN, field_192440_dO, field_192441_dP, field_192442_dQ);


    }


    private static IRecipe getLogPlankRecipe(ItemStack log) {
        if (log.func_190926_b())
            return null;
        for (IRecipe recipe : CraftingManager.field_193380_a) {
            if (InventoryUtils.applyIngredients(recipe.func_192400_c(), log)) {
                if (isPlank(recipe.func_77571_b())) {
                    return recipe;
                }
            }
        }
        return null;
    }

    private static boolean isPlank(ItemStack output) {
        return BWMOreDictionary.listContains(output, OreDictionary.getOres("plankWood"));
    }

    public static void registerOre(String ore, Collection<? extends Block> stacks) {
        stacks.forEach(s -> OreDictionary.registerOre(ore, s));
    }

    public static void registerOre(String ore, List<ItemStack> stacks) {
        stacks.forEach(s -> OreDictionary.registerOre(ore, s));
    }

    public static void registerOre(String ore, ItemStack... items) {
        for (ItemStack i : items)
            OreDictionary.registerOre(ore, i);
    }

    public static void registerOre(String ore, Block... blocks) {
        for (Block block : blocks)
            registerOre(ore, new ItemStack(block));
    }

    public static void registerOre(String ore, Item... items) {
        for (Item item : items)
            registerOre(ore, new ItemStack(item));
    }

    public static void oreGathering() {
        nuggetNames = getOreIngredients("nugget");
        dustNames = getOreIngredients("dust");
        oreNames = getOreIngredients("ore");
        ingotNames = getOreIngredients("ingot");

        VariantUtils.BLOCK_VARIANTS.addAll(Lists.newArrayList(
                BlockVariant.builder()
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150347_e))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150446_ar))
                        .addVariant(WALL, new ItemStack(Blocks.field_150463_bK)),
                BlockVariant.builder()
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150336_V))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150389_bf)),
                BlockVariant.builder()
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150371_ca))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150370_cb)),
                BlockVariant.builder()
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150322_A))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150372_bz)),
                BlockVariant.builder()
                        .addVariant(BLOCK, new ItemStack(Blocks.field_180395_cM))
                        .addVariant(STAIR, new ItemStack(Blocks.field_180396_cN)),
                BlockVariant.builder()
                        .addVariant(BLOCK, new ItemStack(Blocks.field_185767_cT))
                        .addVariant(STAIR, new ItemStack(Blocks.field_185769_cV)),
                BlockVariant.builder()
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150385_bj))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150387_bl))
                        .addVariant(FENCE, new ItemStack(Blocks.field_150386_bk)),
                WoodVariant.builder()
                        .addVariant(LOG, new ItemStack(Blocks.field_150364_r, 1, 0))
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150344_f, 1, 0))
                        .addVariant(FENCE, new ItemStack(Blocks.field_180407_aO))
                        .addVariant(FENCE_GATE, new ItemStack(Blocks.field_180390_bo))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150476_ad)),
                WoodVariant.builder()
                        .addVariant(LOG, new ItemStack(Blocks.field_150364_r, 1, 1))
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150344_f, 1, 1))
                        .addVariant(FENCE, new ItemStack(Blocks.field_180408_aP))
                        .addVariant(FENCE_GATE, new ItemStack(Blocks.field_180391_bp))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150485_bF)),
                WoodVariant.builder()
                        .addVariant(LOG, new ItemStack(Blocks.field_150364_r, 1, 2))
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150344_f, 1, 2))
                        .addVariant(FENCE, new ItemStack(Blocks.field_180404_aQ))
                        .addVariant(FENCE_GATE, new ItemStack(Blocks.field_180392_bq))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150487_bG)),
                WoodVariant.builder()
                        .addVariant(LOG, new ItemStack(Blocks.field_150364_r, 1, 3))
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150344_f, 1, 3))
                        .addVariant(FENCE, new ItemStack(Blocks.field_180403_aR))
                        .addVariant(FENCE_GATE, new ItemStack(Blocks.field_180386_br))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150481_bH)),
                WoodVariant.builder()
                        .addVariant(LOG, new ItemStack(Blocks.field_150363_s, 1, 0))
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150344_f, 1, 4))
                        .addVariant(FENCE, new ItemStack(Blocks.field_180405_aT))
                        .addVariant(FENCE_GATE, new ItemStack(Blocks.field_180387_bt))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150400_ck)),
                WoodVariant.builder()
                        .addVariant(LOG, new ItemStack(Blocks.field_150363_s, 1, 1))
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150344_f, 1, 5))
                        .addVariant(FENCE, new ItemStack(Blocks.field_180406_aS))
                        .addVariant(FENCE_GATE, new ItemStack(Blocks.field_180385_bs))
                        .addVariant(STAIR, new ItemStack(Blocks.field_150401_cl)),
                WoodVariant.builder()
                        .addVariant(LOG, new ItemStack(BWMBlocks.BLOOD_LOG))
                        .addVariant(BLOCK, new ItemStack(Blocks.field_150344_f, 1, 3))
                        .addVariant(IBlockVariants.EnumBlock.SAWDUST, ItemMaterial.getStack(ItemMaterial.EnumMaterial.SOUL_DUST))
        ));
    }

    public static void findLogRecipes() {
        BWMOreDictionary.logs = OreDictionary.getOres("logWood").stream().filter(s -> !s.func_190926_b()).collect(Collectors.toList());
        for (ItemStack log : logs) {
            Item logItem = log.func_77973_b();
            if (logItem.func_77640_w() != null) {
                NonNullList<ItemStack> listSubItems = NonNullList.func_191196_a();
                log.func_77973_b().func_150895_a(logItem.func_77640_w(), listSubItems);
                for (ItemStack subLog : listSubItems) {
                    IRecipe recipe = getLogPlankRecipe(subLog);
                    if (recipe != null) {
                        ItemStack plank = recipe.func_77571_b();
                        if (!plank.func_190926_b()) {
                            logRecipes.add(recipe);
                            if (!isWoodRegistered(log)) {
                                VariantUtils.BLOCK_VARIANTS.add(WoodVariant.builder().addVariant(LOG, subLog).addVariant(BLOCK, plank));
                            }
                        }
                    }
                }
            }
        }
    }

    public static boolean isWoodRegistered(ItemStack stack) {
        return VariantUtils.BLOCK_VARIANTS.stream().anyMatch(wood -> wood.getStack(LOG, 1).func_77969_a(stack));
    }

    public static List<ItemStack> getOreNames(String prefix) {
        return Arrays.stream(OreDictionary.getOreNames()).filter(Objects::nonNull).filter(n -> n.startsWith(prefix)).map(OreDictionary::getOres).filter(o -> !o.isEmpty()).flatMap(Collection::stream).collect(Collectors.toList());
    }

    public static List<Ore> getOreIngredients(String prefix) {
        return Arrays.stream(OreDictionary.getOreNames()).filter(Objects::nonNull).filter(n -> n.startsWith(prefix)).map(n -> new Ore(prefix, n)).collect(Collectors.toList());
    }

    public static boolean isOre(ItemStack stack, String ore) {
        return listContains(stack, OreDictionary.getOres(ore));
    }

    public static boolean listContains(ItemStack check, List<ItemStack> list) {
        if (list != null) {
            if (list.isEmpty()) return false;
            for (ItemStack item : list) {
                if (ItemStack.func_179545_c(check, item) || (check.func_77973_b() == item.func_77973_b() && item.func_77952_i() == OreDictionary.WILDCARD_VALUE)) {
                    return !item.func_77942_o() || ItemStack.func_77970_a(check, item);
                }
            }
        }
        return false;
    }

    public static List<String> getOres(ItemStack stack) {
        return IntStream.of(OreDictionary.getOreIDs(stack)).mapToObj(OreDictionary::getOreName).collect(Collectors.toList());
    }

    public static boolean hasPrefix(ItemStack stack, String suffix) {
        return listContains(stack, getOreNames(suffix));
    }

    public static boolean isToolForOre(String tool, ItemStack stack) {
        return toolEffectiveOre.get(tool).stream().anyMatch(getOres(stack)::contains);
    }

    public static List<Ingredient> fromOres(String... ores) {
        return Arrays.stream(ores).map(OreIngredient::new).collect(Collectors.toList());
    }

    public static class Ore extends OreIngredient {
        private final String prefix;
        private final String ore;

        public Ore(String prefix, String ore) {
            super(ore);
            this.prefix = prefix;
            this.ore = ore;
        }

        public List<ItemStack> getOres() {
            return OreDictionary.getOres(ore);
        }

        public String getOre() {
            return ore;
        }

        public String getPrefix() {
            return prefix;
        }

        public String getSuffix() {
            return ore.substring(getPrefix().length());
        }

    }
}
