package betterwithmods.common.blocks;

import betterwithmods.library.common.block.BlockActiveFacing;
import betterwithmods.library.utils.DirUtils;
import betterwithmods.library.utils.ingredient.blockstate.BlockStateIngredient;
import com.google.common.collect.Sets;
import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.init.SoundEvents;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import javax.annotation.Nonnull;
import java.util.Random;
import java.util.Set;
import java.util.function.Predicate;

public class BlockDetector extends BlockActiveFacing {
    public static Set<IDetection> DETECTION_HANDLERS = Sets.newHashSet();

    public BlockDetector() {
        super(Material.field_151576_e);
        this.func_149711_c(3.5F);
        this.func_149672_a(SoundType.field_185851_d);
        this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(DirUtils.FACING, EnumFacing.NORTH).func_177226_a(ACTIVE, false));
        this.func_149675_a(true);
        this.setHarvestLevel("pickaxe", 0);
    }


    @SuppressWarnings("deprecation")
    @Override
    public boolean isSideSolid(IBlockState base_state, @Nonnull IBlockAccess world, @Nonnull BlockPos pos, EnumFacing side) {
        return true;
    }

    @Override
    public int func_149738_a(World world) {
        return 1;
    }

    @Nonnull
    @Override
    public IBlockState getStateForPlacement(@Nonnull World world, @Nonnull BlockPos pos, @Nonnull EnumFacing side, float flX, float flY, float flZ, int meta, @Nonnull EntityLivingBase entity, EnumHand hand) {
        return withFacing(super.getStateForPlacement(world, pos, side, flX, flY, flZ, meta, entity, hand), EnumFacing.func_190914_a(pos, entity));
    }

    @Override
    public void func_176213_c(World world, BlockPos pos, IBlockState state) {
        super.func_176213_c(world, pos, state);
        setBlockOn(world, pos, false);
        world.func_180497_b(pos, this, func_149738_a(world), 5);
    }

    @SuppressWarnings("deprecation")
    @Override
    public void func_189540_a(IBlockState state, World world, BlockPos pos, Block block, BlockPos other) {
        world.func_180497_b(pos, this, func_149738_a(world), 5);
    }

    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random rand) {
        setBlockOn(world, pos, detectBlock(world, pos));
    }

    @Override
    public boolean canConnectRedstone(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing side) {
        return true;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149744_f(IBlockState state) {
        return state.func_177229_b(ACTIVE);
    }

    @SuppressWarnings("deprecation")
    @Override
    public int func_180656_a(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing facing) {
        return isBlockOn(world, pos) ? 15 : 0;
    }


    public boolean isBlockOn(IBlockAccess world, BlockPos pos) {
        return world.func_180495_p(pos).func_177229_b(ACTIVE);
    }

    public void setBlockOn(World world, BlockPos pos, boolean on) {
        if (on != isBlockOn(world, pos)) {
            IBlockState state = world.func_180495_p(pos).func_177226_a(ACTIVE, on);

            if (on) {
                world.func_184133_a(null, pos, SoundEvents.field_187885_gS, SoundCategory.BLOCKS, 0.3F, 0.7F);
            }

            world.func_175656_a(pos, state);

            for (int i = 0; i < 6; i++) {
                world.func_190524_a(pos.func_177972_a(EnumFacing.func_82600_a(i)), this, pos);
            }
        }
        world.func_180497_b(pos, this, func_149738_a(world), 5);
    }

    public boolean detectBlock(World world, BlockPos pos) {
        EnumFacing facing = getFacing(world.func_180495_p(pos));
        BlockPos offset = pos.func_177972_a(facing);
        for (IDetection detection : DETECTION_HANDLERS) {
            if (detection.apply(facing, world, pos, offset))
                return true;
        }
        return false;
    }


    @Override
    public void nextState(World world, BlockPos pos, IBlockState state) {
        world.func_175656_a(pos, state.func_177226_a(ACTIVE, false).func_177231_a(DirUtils.FACING));
    }

    @Override
    public boolean rotates() {
        return true;
    }

    public interface IDetection {
        boolean apply(EnumFacing direction, World world, BlockPos pos, BlockPos offset);
    }

    public static class EntityDetection implements IDetection {

        @Override
        public boolean apply(EnumFacing direction, World world, BlockPos pos, BlockPos offset) {
            int x = offset.func_177958_n();
            int y = offset.func_177956_o();
            int z = offset.func_177952_p();
            AxisAlignedBB collisionBox = new AxisAlignedBB(x, y, z, x + 1, y + 1, z + 1);
            return world.func_175647_a(Entity.class, collisionBox, Entity::func_70089_S).stream().findAny().isPresent();
        }
    }

    public static class IngredientDetection implements IDetection {
        private BlockStateIngredient ingredients;
        private Predicate<EnumFacing> direction;

        public IngredientDetection(BlockStateIngredient ingredients) {
            this(ingredients, facing -> true);
        }


        public IngredientDetection(BlockStateIngredient ingredients, Predicate<EnumFacing> direction) {
            this.ingredients = ingredients;
            this.direction = direction;
        }

        public boolean apply(EnumFacing direction, World world, BlockPos pos, BlockPos offset) {
            if (this.direction.test(direction)) {
                return ingredients.test(world, offset, world.func_180495_p(offset));
            }
            return false;
        }
    }

}
