package betterwithmods.common.blocks;

import betterwithmods.api.block.ISoulContainer;
import betterwithmods.api.block.IUrnConnector;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.mechanical.mech_machine.BlockFilteredHopper;
import betterwithmods.library.common.block.BlockBase;
import betterwithmods.library.utils.InventoryUtils;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nonnull;
import java.util.Random;

import static net.minecraft.util.EnumFacing.UP;

public class BlockUrn extends BlockBase implements ISoulContainer {
    public static final PropertyBool UNDERHOPPER = PropertyBool.func_177716_a("underhopper");
    private static final double OFFSET = 0.375D;
    private static final AxisAlignedBB URN_AABB = new AxisAlignedBB(0.3125D, 0, 0.3125D, 0.6875D, 0.625D, 0.6875D);
    private static final AxisAlignedBB UNDER_HOPPER_AABB = URN_AABB.func_72317_d(0, OFFSET, 0);

    private final EnumType type;

    public BlockUrn(EnumType type) {
        super(Material.field_151576_e);
        this.type = type;
        this.func_149711_c(2.0F);
        this.func_180632_j(func_176223_P().func_177226_a(UNDERHOPPER, false));
    }

    @Override
    public boolean func_176196_c(World worldIn, @Nonnull BlockPos pos) {
        BlockPos down = pos.func_177977_b();
        BlockFaceShape blockfaceshape = worldIn.func_180495_p(down).func_193401_d(worldIn, down, UP);
        boolean below = blockfaceshape != BlockFaceShape.BOWL && blockfaceshape != BlockFaceShape.UNDEFINED;

        BlockPos up = pos.func_177984_a();
        IBlockState state = worldIn.func_180495_p(up);
        boolean above = state.func_177230_c() instanceof IUrnConnector;
        return below || above;
    }

    @Override
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos) {
        if (!func_176196_c(worldIn, pos)) {
            this.func_176226_b(worldIn, pos, state, 0);
            worldIn.func_175698_g(pos);
        }
        super.func_189540_a(state, worldIn, pos, blockIn, fromPos);
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Nonnull
    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess world, BlockPos pos) {
        return state.func_185899_b(world, pos).func_177229_b(UNDERHOPPER) ? UNDER_HOPPER_AABB : URN_AABB;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void func_180655_c(IBlockState state, World world, BlockPos pos, Random rand) {
        if (type != EnumType.EMPTY && rand.nextInt(64) == 0) {
            int x = pos.func_177958_n();
            int y = pos.func_177956_o();
            int z = pos.func_177952_p();
            float flX = x + rand.nextFloat();
            float flY = y + rand.nextFloat() * 0.5F + 0.625F;
            float flZ = z + rand.nextFloat();
            world.func_184134_a(pos.func_177958_n() + 0.5D, pos.func_177956_o() + 0.5D, pos.func_177952_p() + 0.5D,
                    SoundEvents.field_187551_bH, SoundCategory.BLOCKS, 1.0F, rand.nextFloat() * 0.1F + 0.45F,
                    false);
            world.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, flX, flY, flZ, 0.0D, 0.0D, 0.0D);
        }
    }

    @Nonnull
    @Override
    public IBlockState func_176221_a(@Nonnull IBlockState state, IBlockAccess world, BlockPos pos) {
        BlockPos up = pos.func_177984_a();
        Block block = world.func_180495_p(up).func_177230_c();
        if (block instanceof BlockFilteredHopper) {
            return state.func_177226_a(UNDERHOPPER, true);
        }
        return state;
    }

    @Override
    public IProperty<?>[] getProperties() {
        return new IProperty[]{UNDERHOPPER};
    }

    @Nonnull
    @Override
    public IBlockState func_176203_a(int meta) {
        return func_176223_P().func_177226_a(UNDERHOPPER, meta == 1);
    }

    @Nonnull
    @Override
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face) {
        return face == EnumFacing.DOWN ? BlockFaceShape.CENTER : BlockFaceShape.UNDEFINED;
    }

    @Override
    public int func_176201_c(IBlockState state) {
        return state.func_177229_b(UNDERHOPPER) ? 1 : 0;
    }

    @Override
    public int getMaxSouls() {
        return type.getSouls();
    }

    @Override
    public boolean onFull(World world, BlockPos pos) {
        switch (type) {
            case EMPTY:
                if (world.func_175698_g(pos)) {
                    InventoryUtils.ejectStackWithOffset(world, pos, new ItemStack(BWMBlocks.SOUL_URN));
                }
                break;
            default:
                break;
        }
        return true;
    }

    public enum EnumType {
        EMPTY(8),
        SOUL(0);

        final int souls;

        EnumType(int souls) {
            this.souls = souls;
        }

        public int getSouls() {
            return souls;
        }
    }

}
