package betterwithmods.common.event;

import betterwithmods.lib.ModLib;
import betterwithmods.module.internal.MiscRegistry;
import betterwithmods.module.internal.SoundRegistry;
import betterwithmods.util.PlayerUtils;
import it.unimi.dsi.fastutil.objects.Object2IntMap;
import it.unimi.dsi.fastutil.objects.Object2IntOpenHashMap;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.SoundCategory;
import net.minecraft.world.World;
import net.minecraftforge.event.entity.living.LivingAttackEvent;
import net.minecraftforge.event.entity.living.LivingEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.eventhandler.Event;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;

import java.util.UUID;

@Mod.EventBusSubscriber(modid = ModLib.MODID)
public class PenaltyEventHandler {

    private static Object2IntMap<UUID> painTimers = new Object2IntOpenHashMap<>();

    @SubscribeEvent
    public static void onJump(LivingEvent.LivingJumpEvent event) {
        //This has to run on clientside and serverside
        if (event.getEntityLiving() instanceof EntityPlayer) {
            EntityPlayer player = (EntityPlayer) event.getEntityLiving();
            //Whether the player can jump.
            if (!MiscRegistry.PENALTY_HANDLERS.canJump(player)) {
                event.getEntityLiving().field_70159_w = 0;
                event.getEntityLiving().field_70181_x = 0;
                event.getEntityLiving().field_70179_y = 0;
            }
        }
    }

    @SubscribeEvent
    public static void onPlayerTick(TickEvent.PlayerTickEvent event) {
        if (event.phase == TickEvent.Phase.START)
            return;

        EntityPlayer player = event.player;
        //Don't run on client side
        if (player.field_70170_p.field_72995_K)
            return;

        if (!PlayerUtils.isSurvival(player))
            return;

        //Handle whether the player can sprint
        if (!MiscRegistry.PENALTY_HANDLERS.canSprint(player)) {
            player.func_70031_b(false);
        }

        //Swimming
        if (player.func_70090_H() && !MiscRegistry.PENALTY_HANDLERS.canSwim(player)) {
            if (!PlayerUtils.isNearBottom(player)) {
                player.field_70181_x -= 0.04;
            }
        }
    }

    private static boolean inPain(EntityPlayer player) {
        UUID uuid = player.func_110124_au();
        if (uuid == null)
            return false;
        if (painTimers.getOrDefault(uuid, 0) > 60) {
            painTimers.put(uuid, 0);
            return true;
        }
        painTimers.put(uuid, painTimers.getOrDefault(uuid, 0) + 1);
        return false;
    }

    @SubscribeEvent
    public static void onPlayerUpdate(LivingEvent.LivingUpdateEvent event) {
        if (event.getEntityLiving() instanceof EntityPlayer) {
            EntityPlayer player = (EntityPlayer) event.getEntityLiving();
            World world = player.field_70170_p;

            if (!PlayerUtils.isSurvival(player) || player.func_184218_aH()) {
                //Remove the modifier when gamemode changes.
                PlayerUtils.removeModifier(player, SharedMonsterAttributes.field_111263_d, PlayerUtils.PENALTY_SPEED_UUID);
                return;
            }
            //Speed
            double speed = MiscRegistry.PENALTY_HANDLERS.getSpeedModifier(player);
            if (speed != 0) {
                PlayerUtils.changeSpeed(player, "Penalty Speed Modifier", speed, PlayerUtils.PENALTY_SPEED_UUID);
            }

            //Pain
            if (!world.field_72995_K && MiscRegistry.PENALTY_HANDLERS.inPain(player)) {
                if (PlayerUtils.isMoving(player) && inPain(player)) {
                    world.func_184133_a(null, player.func_180425_c(), SoundRegistry.ENTITY_PLAYER_OOF, SoundCategory.BLOCKS, 0.75f, 1f);
                }
            }

        }
    }

    @SubscribeEvent
    public static void onPlayerAttack(LivingAttackEvent event) {
        if (event.getSource().func_76346_g() instanceof EntityPlayer) {
            EntityPlayer player = (EntityPlayer) event.getSource().func_76346_g();
            if (PlayerUtils.isSurvival(player)) {
                if (!MiscRegistry.PENALTY_HANDLERS.canAttack(player)) {
                    player.func_184185_a(SoundRegistry.ENTITY_PLAYER_OOF, 0.75f, 1f);
                    event.setCanceled(true);
                    event.setResult(Event.Result.DENY);
                }
            }
        }
    }
}
