package betterwithmods.module.compat.jei;

import betterwithmods.api.recipe.output.IOutput;
import betterwithmods.api.recipe.output.impl.ChanceOutput;
import betterwithmods.api.recipe.output.impl.RandomOutput;
import betterwithmods.api.recipe.output.impl.StackOutput;
import betterwithmods.client.gui.GuiSteelAnvil;
import betterwithmods.client.gui.bulk.GuiCauldron;
import betterwithmods.client.gui.bulk.GuiCrucible;
import betterwithmods.client.gui.bulk.GuiFilteredHopper;
import betterwithmods.client.gui.bulk.GuiMill;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWMItems;
import betterwithmods.common.container.anvil.ContainerSteelAnvil;
import betterwithmods.common.container.bulk.ContainerMill;
import betterwithmods.common.items.ItemMaterial;
import betterwithmods.common.registry.anvil.AnvilRecipe;
import betterwithmods.common.registry.block.recipe.KilnRecipe;
import betterwithmods.common.registry.block.recipe.SawRecipe;
import betterwithmods.common.registry.block.recipe.TurntableRecipe;
import betterwithmods.common.registry.bulk.recipes.CauldronRecipe;
import betterwithmods.common.registry.bulk.recipes.CrucibleRecipe;
import betterwithmods.common.registry.bulk.recipes.MillRecipe;
import betterwithmods.common.registry.crafting.ToolBaseRecipe;
import betterwithmods.common.registry.crafting.ToolDamageRecipe;
import betterwithmods.common.registry.heat.BWMHeatRegistry;
import betterwithmods.common.registry.hopper.recipes.HopperRecipe;
import betterwithmods.module.compat.jei.category.*;
import betterwithmods.module.compat.jei.ingredient.OutputHelper;
import betterwithmods.module.compat.jei.ingredient.OutputRenderer;
import betterwithmods.module.compat.jei.wrapper.*;
import betterwithmods.module.internal.RecipeRegistry;
import betterwithmods.module.recipes.miniblocks.DynblockUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import mezz.jei.Internal;
import mezz.jei.api.*;
import mezz.jei.api.ingredients.IModIngredientRegistration;
import mezz.jei.api.recipe.IRecipeCategoryRegistration;
import mezz.jei.api.recipe.IVanillaRecipeFactory;
import mezz.jei.api.recipe.VanillaRecipeCategoryUid;
import mezz.jei.api.recipe.transfer.IRecipeTransferRegistry;
import mezz.jei.plugins.vanilla.crafting.ShapelessRecipeWrapper;
import mezz.jei.startup.StackHelper;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraftforge.common.crafting.IShapedRecipe;

import javax.annotation.Nonnull;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Consumer;

@mezz.jei.api.JEIPlugin
public class JEI implements IModPlugin {
    public static final List<Class<? extends IOutput>> ALL_OUTPUTS = Lists.newArrayList();
    public static IJeiHelpers HELPER;
    public static IJeiRuntime JEI_RUNTIME;

    static {
        ALL_OUTPUTS.add(IOutput.class);
        ALL_OUTPUTS.add(StackOutput.class);
        ALL_OUTPUTS.add(RandomOutput.class);
        ALL_OUTPUTS.add(ChanceOutput.class);
    }

    public static void doAllOutputs(Consumer<Class<? extends IOutput>> consumer) {
        ALL_OUTPUTS.forEach(consumer);
    }

    private static String getHeatName(String base, int heat) {
        return String.format("%s.%d", base, heat);
    }

    @Override
    public void onRuntimeAvailable(IJeiRuntime jeiRuntime) {
        if (JEI_RUNTIME == null) {
            JEI_RUNTIME = jeiRuntime;
        }
    }

    @Override
    public void registerCategories(IRecipeCategoryRegistration reg) {
        final IJeiHelpers helpers = reg.getJeiHelpers();
        final IGuiHelper guiHelper = helpers.getGuiHelper();

        for (int heat : BWMHeatRegistry.allHeatLevels()) {
            reg.addRecipeCategories(
                    new CookingPotRecipeCategory(guiHelper, getHeatName(JEILib.CAULDRON_BASE, heat)),
                    new CookingPotRecipeCategory(guiHelper, getHeatName(JEILib.CRUCIBLE_BASE, heat)),
                    new KilnRecipeCategory(guiHelper, getHeatName(JEILib.KILN_BASE, heat))
            );
        }

        reg.addRecipeCategories(
                new MillRecipeCategory(guiHelper),
                new SawRecipeCategory(guiHelper),
                new TurntableRecipeCategory(guiHelper),
                new HopperRecipeCategory(guiHelper),
                new SteelAnvilRecipeCategory(guiHelper)
        );
    }

    @Override
    public void registerItemSubtypes(ISubtypeRegistry subtypeRegistry) {
        subtypeRegistry.useNbtForSubtypes(getAllMiniBlocks());
        subtypeRegistry.useNbtForSubtypes(BWMItems.BARK);
    }

    private Item[] getAllMiniBlocks() {
        return DynblockUtils.blocks().stream().map(Item::func_150898_a).toArray(Item[]::new);
    }

    @Override
    public void registerIngredients(IModIngredientRegistration registry) {
        StackHelper stackHelper = Internal.getStackHelper();
        doAllOutputs(clazz -> registry.register(JEILib.IOUTPUT, Collections.emptySet(), new OutputHelper<>(stackHelper), new OutputRenderer<>()));
    }

    @Override
    public void register(@Nonnull IModRegistry reg) {
        HELPER = reg.getJeiHelpers();

        registerHeatBasedRecipes(reg);

        reg.handleRecipes(MillRecipe.class, r -> new BulkRecipeWrapper<>(HELPER, r, 3), JEILib.MILLSTONE_UID);
        reg.handleRecipes(SawRecipe.class, r -> new BlockRecipeWrapper<>(HELPER, r, 3), JEILib.SAW_UID);
        reg.handleRecipes(TurntableRecipe.class, recipe -> new TurntableRecipeWrapper(HELPER, recipe), JEILib.TURNTABLE_UID);
        reg.handleRecipes(HopperRecipe.class, recipe -> new HopperRecipeWrapper(HELPER, recipe), JEILib.HOPPER_UID);
        reg.handleRecipes(AnvilRecipe.class, recipe -> {
            IRecipe r = recipe.getRecipe();
            if (r instanceof IShapedRecipe) {
                return new ShapedAnvilRecipeWrapper(HELPER, (IShapedRecipe) r);
            } else {
                return new ShapelessRecipeWrapper<>(HELPER, r);
            }
        }, JEILib.ANVIL_UID);
        reg.handleRecipes(ToolBaseRecipe.class, recipe -> new ShapelessRecipeWrapper<>(HELPER, recipe), VanillaRecipeCategoryUid.CRAFTING);
        reg.handleRecipes(ToolDamageRecipe.class, recipe -> new ShapelessRecipeWrapper<>(HELPER, recipe), VanillaRecipeCategoryUid.CRAFTING);

        reg.addRecipes(RecipeRegistry.MILLSTONE.getValuesCollection(), JEILib.MILLSTONE_UID);
        reg.addRecipes(RecipeRegistry.WOOD_SAW.getDisplayRecipes(), JEILib.SAW_UID);
        reg.addRecipes(RecipeRegistry.TURNTABLE.getDisplayRecipes(), JEILib.TURNTABLE_UID);
        reg.addRecipes(RecipeRegistry.FILTERED_HOPPER.getDisplayRecipes(), JEILib.HOPPER_UID);
        reg.addRecipes(RecipeRegistry.ANVIL.getDisplayRecipes(), JEILib.ANVIL_UID);

        reg.addRecipeCatalyst(new ItemStack(BWMBlocks.MILLSTONE), JEILib.MILLSTONE_UID);
        reg.addRecipeCatalyst(new ItemStack(BWMBlocks.FILTERED_HOPPER), JEILib.HOPPER_UID);
        reg.addRecipeCatalyst(new ItemStack(BWMBlocks.TURNTABLE), JEILib.TURNTABLE_UID);

        reg.addRecipeCatalyst(new ItemStack(BWMBlocks.SAW), JEILib.SAW_UID);
        reg.addRecipeCatalyst(new ItemStack(BWMBlocks.STEEL_ANVIL), JEILib.ANVIL_UID);

        reg.addRecipeClickArea(GuiMill.class, 81, 19, 14, 14, JEILib.MILLSTONE_UID);
        reg.addRecipeClickArea(GuiSteelAnvil.class, 88, 41, 28, 23, JEILib.ANVIL_UID);
        reg.addRecipeClickArea(GuiFilteredHopper.class, 81, 19, 14, 14, JEILib.HOPPER_UID);

        registerAnvil(reg);

        IRecipeTransferRegistry recipeTransferRegistry = reg.getRecipeTransferRegistry();


        recipeTransferRegistry.addRecipeTransferHandler(new CraftingTransfer<>(JEILib.ANVIL_UID, ContainerSteelAnvil.class));
        recipeTransferRegistry.addRecipeTransferHandler(new CraftingTransfer<>(JEILib.MILLSTONE_UID, ContainerMill.class));
        recipeTransferRegistry.addRecipeTransferHandler(new AnvilVanillaCraftingTransfer());

    }

    private void registerAnvil(IModRegistry reg) {
        List<ItemStack> tools = Lists.newArrayList(new ItemStack(BWMItems.STEEL_AXE), new ItemStack(BWMItems.STEEL_BATTLEAXE), new ItemStack(BWMItems.STEEL_BOOTS), new ItemStack(BWMItems.STEEL_CHEST), new ItemStack(BWMItems.STEEL_HELMET), new ItemStack(BWMItems.STEEL_HOE), new ItemStack(BWMItems.STEEL_MATTOCK),
                new ItemStack(BWMItems.STEEL_PANTS), new ItemStack(BWMItems.STEEL_PICKAXE), new ItemStack(BWMItems.STEEL_SHOVEL), new ItemStack(BWMItems.STEEL_SWORD));
        IVanillaRecipeFactory v = reg.getJeiHelpers().getVanillaRecipeFactory();
        for (ItemStack stack : tools) {
            ItemStack dam1 = stack.func_77946_l();
            dam1.func_77964_b(dam1.func_77958_k());
            ItemStack dam2 = stack.func_77946_l();
            dam2.func_77964_b(dam2.func_77958_k() * 3 / 4);
            ItemStack dam3 = stack.func_77946_l();
            dam3.func_77964_b(dam3.func_77958_k() * 2 / 4);

            v.createAnvilRecipe(dam1, Collections.singletonList(ItemMaterial.getStack(ItemMaterial.EnumMaterial.STEEL_INGOT)), Collections.singletonList(dam2));
            v.createAnvilRecipe(dam2, Collections.singletonList(ItemMaterial.getStack(ItemMaterial.EnumMaterial.STEEL_INGOT)), Collections.singletonList(dam3));
        }

    }

    private void registerHeatBasedRecipes(@Nonnull IModRegistry reg) {
        Set<String>
                cauldron = Sets.newHashSet(),
                crucible = Sets.newHashSet(),
                kiln = Sets.newHashSet();

        IRecipeTransferRegistry recipeTransferRegistry = reg.getRecipeTransferRegistry();

        for (int heat : BWMHeatRegistry.allHeatLevels()) {
            String cauldronUID = JEILib.uid(getHeatName(JEILib.CAULDRON_BASE, heat));
            String crucibleUID = JEILib.uid(getHeatName(JEILib.CRUCIBLE_BASE, heat));
            String kilnUID = JEILib.uid(getHeatName(JEILib.KILN_BASE, heat));
            cauldron.add(cauldronUID);
            crucible.add(crucibleUID);
            kiln.add(kilnUID);

            reg.handleRecipes(CauldronRecipe.class, recipe -> new BulkRecipeWrapper<>(HELPER, recipe, 9), cauldronUID);
            reg.handleRecipes(CrucibleRecipe.class, recipe -> new BulkRecipeWrapper<>(HELPER, recipe, 9), crucibleUID);
            reg.handleRecipes(KilnRecipe.class, recipe -> new KilnRecipeWrapper(HELPER, recipe), kilnUID);

            reg.addRecipes(RecipeRegistry.CAULDRON.getRecipesForHeat(heat), cauldronUID);
            reg.addRecipes(RecipeRegistry.CRUCIBLE.getRecipesForHeat(heat), crucibleUID);
            reg.addRecipes(RecipeRegistry.KILN.getRecipesForHeat(heat), kilnUID);
        }

        reg.addRecipeCatalyst(new ItemStack(BWMBlocks.CAULDRON), cauldron.toArray(new String[0]));
        reg.addRecipeCatalyst(new ItemStack(BWMBlocks.CRUCIBLE), crucible.toArray(new String[0]));
        reg.addRecipeCatalyst(new ItemStack(Blocks.field_150336_V), kiln.toArray(new String[0]));

        reg.addRecipeClickArea(GuiCauldron.class, 81, 19, 14, 14, cauldron.toArray(new String[0]));
        reg.addRecipeClickArea(GuiCrucible.class, 81, 19, 14, 14, crucible.toArray(new String[0]));

    }


}

