package betterwithmods.module.tweaks;

import betterwithmods.common.entity.EntityJungleSpider;
import betterwithmods.common.entity.SpawningWhitelist;
import betterwithmods.library.common.modularity.impl.Feature;
import betterwithmods.library.utils.ingredient.blockstate.MaterialIngredient;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EnumCreatureType;
import net.minecraft.entity.monster.EntitySlime;
import net.minecraft.entity.monster.EntityWitch;
import net.minecraft.init.Blocks;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.DimensionType;
import net.minecraft.world.World;
import net.minecraft.world.biome.Biome;
import net.minecraftforge.common.BiomeDictionary;
import net.minecraftforge.event.entity.living.LivingSpawnEvent;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.Event;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.registry.EntityRegistry;

/**
 * Created by primetoxinz on 4/20/17.
 */


public class MobSpawning extends Feature {
    public static final SpawningWhitelist NETHER = new SpawningWhitelist();
    public static final SpawningWhitelist SLIME = new SpawningWhitelist();

    private static boolean slime, nether, witches, jungleSpiders;

    @SubscribeEvent
    public void denySlimeSpawns(LivingSpawnEvent.CheckSpawn event) {
        if (event.getResult() == Event.Result.ALLOW)
            return;
        if (!slime)
            return;
        World world = event.getWorld();
        if (world != null && world.field_73011_w.func_186058_p() == DimensionType.OVERWORLD) {
            if (event.getEntityLiving() instanceof EntitySlime) {
                BlockPos pos = new BlockPos(event.getEntity().field_70165_t, event.getEntity().field_70163_u - 1, event.getEntity().field_70161_v);
                if (SLIME.isBlacklisted(world, pos, world.func_180495_p(pos)))
                    event.setResult(Event.Result.DENY);
            }
        }
    }

    @SubscribeEvent
    public void denyNetherSpawns(LivingSpawnEvent.CheckSpawn event) {
        if (event.getResult() == Event.Result.ALLOW)
            return;
        if (!nether)
            return;
        World world = event.getWorld();
        if (world != null && world.field_73011_w.func_186058_p() == DimensionType.NETHER) {
            if (event.getEntityLiving().isCreatureType(EnumCreatureType.MONSTER, false)) {
                double monX = event.getEntity().field_70165_t;
                double monY = event.getEntity().field_70163_u;
                double monZ = event.getEntity().field_70161_v;
                int x = MathHelper.func_76128_c(monX);
                int y = MathHelper.func_76128_c(monY);
                int z = MathHelper.func_76128_c(monZ);
                BlockPos pos = new BlockPos(x, y - 1, z);
                IBlockState state = world.func_180495_p(pos);
                if (!world.func_175623_d(pos) && NETHER.isBlacklisted(world, pos, state))
                    event.setResult(Event.Result.DENY);
            }
        }
    }

    @Override
    public void onInit(FMLInitializationEvent event) {

        slime = loadProperty("Limit Slime Spawning", true).setComment("Slimes can only spawn on natural blocks").get();
        nether = loadProperty("Limit Nether Spawning", true).setComment("Nether Mobs can only spawn on nether blocks").get();
        witches = loadProperty("Limit Witch Spawning", true).setComment("Witches can only spawn in swamps").get();
        jungleSpiders = loadProperty("Jungle Spider Spawning", true).setComment("Jungle Spiders can spawn in jungles").get();

        if (nether) {
            NETHER.addBlock(Blocks.field_150424_aL);
            NETHER.addBlock(Blocks.field_150385_bj);
            NETHER.addBlock(Blocks.field_150425_aM);
            NETHER.addBlock(Blocks.field_150351_n);
            NETHER.addBlock(Blocks.field_150371_ca);
        }

        if (slime) {
            SLIME.addIngredient(new MaterialIngredient(Material.field_151577_b, Material.field_151576_e, Material.field_151578_c));
        }

        for (Biome biome : Biome.field_185377_q) {
            if (jungleSpiders && BiomeDictionary.hasType(biome, BiomeDictionary.Type.JUNGLE)) {
                EntityRegistry.addSpawn(EntityJungleSpider.class, 100, 1, 3, EnumCreatureType.MONSTER, biome);
            }
            if (witches && !BiomeDictionary.hasType(biome, BiomeDictionary.Type.SWAMP)) {
                EntityRegistry.removeSpawn(EntityWitch.class, EnumCreatureType.MONSTER, biome);
            }
        }
    }

    @Override
    public String getDescription() {
        return "Nether Mobs can only spawn on nether blocks and Slimes can only spawn on natural blocks. Also adjusts whether witches only spawn in swamps and if jungle spiders spawn in jungles.";
    }

    @Override
    public boolean hasEvent() {
        return true;
    }

}
